==> de.bsvrz.buv.plugin.dobj.source_4.0.0.201905071350.jar: de/bsvrz/buv/plugin/dobj/editors/Splitter.java <==
/*
 * Rahmenwerk-Plug-in "Darstellungsobjekte"
 * Copyright (C) 2018 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */
package de.bsvrz.buv.plugin.dobj.editors;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;

import org.eclipse.draw2d.ColorConstants;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Layout;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Sash;

class Splitter extends Composite {

	// FIXME: Ohne Kopieren der Klasse auskommen

	public static final int DEFAULT_SASH_WIDTH = 5;
	private static final int DRAG_MINIMUM = 62;
	private static final String MAINTAIN_SIZE = "maintain size"; //$NON-NLS-1$

	private int fixedSize = 150;
	private int orientation = SWT.VERTICAL;
	private Sash[] sashes = new Sash[0];
	private Control[] controls = new Control[0];
	private Control maxControl;
	private final Listener sashListener;
	private int sashWidth = DEFAULT_SASH_WIDTH;

	/**
	 * PropertyChangeSupport
	 */
	private final PropertyChangeSupport listeners = new PropertyChangeSupport(this);

	public void addFixedSizeChangeListener(final PropertyChangeListener listener) {
		listeners.addPropertyChangeListener(listener);
	}

	protected void firePropertyChange(final int oldValue, final int newValue) {
		listeners.firePropertyChange(MAINTAIN_SIZE, oldValue, newValue);
	}

	public void removeFixedSizeChangeListener(final PropertyChangeListener listener) {
		listeners.removePropertyChangeListener(listener);
	}

	public int getFixedSize() {
		return fixedSize;
	}

	public void setFixedSize(final int newSize) {
		if (newSize == fixedSize) {
			return;
		}

		final int oldSize = fixedSize;
		fixedSize = newSize;
		firePropertyChange(oldSize, fixedSize);
	}

	class SashPainter implements Listener {
		@Override
		public void handleEvent(final Event e) {
			paint((Sash) e.widget, e.gc);
		}
	}

	public Splitter(final Composite parent, final int style) {
		super(parent, checkStyle(style));

		if ((style & SWT.HORIZONTAL) != 0) {
			orientation = SWT.HORIZONTAL;
		}

		addListener(SWT.Resize, new Listener() {
			@Override
			public void handleEvent(final Event e) {
				layout(true);
			}
		});

		sashListener = new Listener() {
			@Override
			public void handleEvent(final Event e) {
				onDragSash(e);
			}
		};
	}

	private static int checkStyle(final int style) {
		final int mask = SWT.BORDER;
		return style & mask;
	}

	@Override
	public Point computeSize(final int wHint, final int hHint, final boolean changed) {

		final Control[] localControls = getControls(true);
		if (localControls.length == 0) {
			return new Point(wHint, hHint);
		}

		int width = 0;
		int height = 0;
		final boolean vertical = (getStyle() & SWT.VERTICAL) != 0;
		if (vertical) {
			width = wHint;
			height += (localControls.length - 1) * getSashWidth();
		} else {
			height = hHint;
			width += localControls.length * getSashWidth();
		}
		for (int i = 0; i < localControls.length; i++) {
			if (vertical) {
				final Point size = localControls[i].computeSize(wHint, SWT.DEFAULT);
				height += size.y;
			} else {
				final Point size = localControls[i].computeSize(SWT.DEFAULT, hHint);
				if (localControls[i].getData(MAINTAIN_SIZE) != null) {
					size.x = fixedSize;
				}
				width += size.x;
			}
		}
		// if (wHint != SWT.DEFAULT) width = wHint;
		// if (hHint != SWT.DEFAULT) height = hHint;

		return new Point(width, height);
	}

	/**
	 * Answer SWT.HORIZONTAL if the controls in the SashForm are laid out side
	 * by side. Answer SWT.VERTICAL if the controls in the SashForm are laid out
	 * top to bottom.
	 */
	@Override
	public int getOrientation() {
		return orientation;
	}

	public int getSashWidth() {
		return sashWidth;
	}

	/**
	 * Answer the control that currently is maximized in the SashForm. This
	 * value may be null.
	 */
	public Control getMaximizedControl() {
		return maxControl;
	}

	private Control[] getControls(final boolean onlyVisible) {
		final Control[] children = getChildren();
		Control[] localControls = new Control[0];
		for (int i = 0; i < children.length; i++) {
			if (children[i] instanceof Sash) {
				continue;
			}
			if (onlyVisible && !children[i].getVisible()) {
				continue;
			}

			final Control[] newControls = new Control[localControls.length + 1];
			System.arraycopy(localControls, 0, newControls, 0, localControls.length);
			newControls[localControls.length] = children[i];
			localControls = newControls;
		}
		return localControls;
	}

	@Override
	public void layout(final boolean changed) {
		final Rectangle area = getClientArea();
		if (area.width == 0 || area.height == 0) {
			return;
		}

		final Control[] newControls = getControls(true);
		if (controls.length == 0 && newControls.length == 0) {
			return;
		}
		controls = newControls;

		if (maxControl != null && !maxControl.isDisposed()) {
			for (int i = 0; i < controls.length; i++) {
				if (controls[i] != maxControl) {
					controls[i].setBounds(-200, -200, 0, 0);
				} else {
					controls[i].setBounds(area);
				}
			}
			return;
		}

		// keep just the right number of sashes
		if (sashes.length < controls.length - 1) {
			final Sash[] newSashes = new Sash[controls.length - 1];
			System.arraycopy(sashes, 0, newSashes, 0, sashes.length);
			final int sashOrientation = (orientation == SWT.HORIZONTAL) ? SWT.VERTICAL : SWT.HORIZONTAL;
			for (int i = sashes.length; i < newSashes.length; i++) {
				newSashes[i] = new Sash(this, sashOrientation);
				newSashes[i].setBackground(ColorConstants.button);
				newSashes[i].addListener(SWT.Paint, new SashPainter());
				newSashes[i].addListener(SWT.Selection, sashListener);
			}
			sashes = newSashes;
		}
		if (sashes.length > controls.length - 1) {
			if (controls.length == 0) {
				for (int i = 0; i < sashes.length; i++) {
					sashes[i].dispose();
				}
				sashes = new Sash[0];
			} else {
				final Sash[] newSashes = new Sash[controls.length - 1];
				System.arraycopy(sashes, 0, newSashes, 0, newSashes.length);
				for (int i = controls.length - 1; i < sashes.length; i++) {
					sashes[i].dispose();
				}
				sashes = newSashes;
			}
		}

		if (controls.length == 0) {
			return;
		}

		// @TODO
		// This only works if the orientation is horizontal, there are two
		// children and one
		// of them has been set to maintain its size.
		int x = area.x;
		int width;
		for (int i = 0; i < controls.length; i++) {
			final Control control = controls[i];
			if (control.getData(MAINTAIN_SIZE) != null) {
				width = fixedSize;
				if (width > area.width) {
					width = area.width - getSashWidth();
				}
				control.setBounds(x, area.y, width, area.height);
				x += width + getSashWidth();
			} else {
				width = Math.max(area.width - fixedSize - getSashWidth(), 0);
				control.setBounds(x, area.y, width, area.height);
				x += width + getSashWidth();
			}
		}
		if (sashes.length > 0) {
			sashes[0].setBounds(controls[0].getBounds().x + controls[0].getBounds().width, area.y, getSashWidth(),
					area.height);
		}
	}

	public void maintainSize(final Control c) {
		final Control[] internalControls = getControls(false);
		for (int i = 0; i < internalControls.length; i++) {
			final Control ctrl = internalControls[i];
			if (ctrl == c) {
				ctrl.setData(MAINTAIN_SIZE, true);
				break;
			}
		}
	}

	void paint(final Sash sash, final GC gc) {
		if (getSashWidth() == 0) {
			return;
		}
		final Point size = sash.getSize();
		if (getOrientation() == SWT.HORIZONTAL) {
			gc.setForeground(ColorConstants.buttonDarker);
			gc.drawLine(getSashWidth() - 1, 0, getSashWidth() - 1, size.y);
			gc.setForeground(ColorConstants.buttonLightest);
			gc.drawLine(0, 0, 0, size.y);
		} else {
			gc.setForeground(ColorConstants.buttonDarker);
			gc.drawLine(0, 0, size.x, 0);
			gc.drawLine(0, getSashWidth() - 1, size.x, getSashWidth() - 1);
			gc.setForeground(ColorConstants.buttonLightest);
			gc.drawLine(0, 1, size.x, 1);
		}
	}

	private void onDragSash(final Event event) {
		if (event.detail == SWT.DRAG) {
			// constrain feedback
			final Rectangle area = getClientArea();
			if (orientation == SWT.HORIZONTAL) {
				if (controls[0].getData(MAINTAIN_SIZE) != null) {
					event.x = Math.max(event.x, DRAG_MINIMUM);
				} else {
					event.x = Math.min(event.x, area.width - DRAG_MINIMUM - getSashWidth());
				}
			} else {
				event.y = Math.min(event.y, area.height - DRAG_MINIMUM - getSashWidth());
			}
			return;
		}

		final Sash sash = (Sash) event.widget;
		int sashIndex = -1;
		for (int i = 0; i < sashes.length; i++) {
			if (sashes[i] == sash) {
				sashIndex = i;
				break;
			}
		}
		if (sashIndex == -1) {
			return;
		}

		final Control c1 = controls[sashIndex];
		final Control c2 = controls[sashIndex + 1];
		final Rectangle b1 = c1.getBounds();
		final Rectangle b2 = c2.getBounds();
		controls = getControls(false);

		final Rectangle sashBounds = sash.getBounds();
		if (orientation == SWT.HORIZONTAL) {
			final int shift = event.x - sashBounds.x;
			b1.width += shift;
			b2.x += shift;
			b2.width -= shift;
		} else {
			final int shift = event.y - sashBounds.y;
			b1.height += shift;
			b2.y += shift;
			b2.height -= shift;
		}

		c1.setBounds(b1);
		sash.setBounds(event.x, event.y, event.width, event.height);
		c2.setBounds(b2);
		// @TODO
		// This will only work when you have two controls, one of whom is set to
		// maintain size
		if (c1.getData(MAINTAIN_SIZE) != null) {
			setFixedSize(c1.getBounds().width);
		} else {
			setFixedSize(c2.getBounds().width);
		}
	}

	/**
	 * If orientation is SWT.HORIZONTAL, lay the controls in the SashForm out
	 * side by side. If orientation is SWT.VERTICAL, lay the controls in the
	 * SashForm out top to bottom.
	 */
	@Override
	public void setOrientation(final int orientation) {
		if (this.orientation == orientation) {
			return;
		}
		if (orientation != SWT.HORIZONTAL && orientation != SWT.VERTICAL) {
			SWT.error(SWT.ERROR_INVALID_ARGUMENT);
		}
		this.orientation = orientation;

		final int sashOrientation = (orientation == SWT.HORIZONTAL) ? SWT.VERTICAL : SWT.HORIZONTAL;
		for (int i = 0; i < sashes.length; i++) {
			sashes[i].dispose();
			sashes[i] = new Sash(this, sashOrientation);
			sashes[i].setBackground(ColorConstants.buttonLightest);
			sashes[i].addListener(SWT.Selection, sashListener);
		}
		layout();
	}

	public void setSashWidth(final int width) {
		sashWidth = width;
	}

	@Override
	public void setLayout(final Layout layout) {
		// SashForm does not use Layout
	}

	/**
	 * Specify the control that should take up the entire client area of the
	 * SashForm. If one control has been maximized, and this method is called
	 * with a different control, the previous control will be minimized and the
	 * new control will be maximized.. if the value of control is null, the
	 * SashForm will minimize all controls and return to the default layout
	 * where all controls are laid out separated by sashes.
	 */
	public void setMaximizedControl(final Control control) {
		if (control == null) {
			if (maxControl != null) {
				maxControl = null;
				layout();
				for (int i = 0; i < sashes.length; i++) {
					sashes[i].setVisible(true);
				}
			}
			return;
		}

		for (int i = 0; i < sashes.length; i++) {
			sashes[i].setVisible(false);
		}
		maxControl = control;
		layout();

	}

}

==> de.bsvrz.buv.plugin.dobj.source_4.5.0.202510170820.jar: de/bsvrz/buv/plugin/dobj/editors/Splitter.java <==
/*
 * Rahmenwerk-Plug-in "Darstellungsobjekte"
 * Copyright (C) 2023 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */
package de.bsvrz.buv.plugin.dobj.editors;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;

import org.eclipse.draw2d.ColorConstants;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Layout;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Sash;

class Splitter extends Composite {

	// FIXME: Ohne Kopieren der Klasse auskommen

	public static final int DEFAULT_SASH_WIDTH = 5;
	private static final int DRAG_MINIMUM = 62;
	private static final String MAINTAIN_SIZE = "maintain size"; //$NON-NLS-1$

	private int fixedSize = 150;
	private int orientation = SWT.VERTICAL;
	private Sash[] sashes = {};
	private Control[] controls = {};
	private Control maxControl;
	private final Listener sashListener;
	private int sashWidth = DEFAULT_SASH_WIDTH;

	/**
	 * PropertyChangeSupport
	 */
	private final PropertyChangeSupport listeners = new PropertyChangeSupport(this);

	public void addFixedSizeChangeListener(final PropertyChangeListener listener) {
		listeners.addPropertyChangeListener(listener);
	}

	protected void firePropertyChange(final int oldValue, final int newValue) {
		listeners.firePropertyChange(MAINTAIN_SIZE, oldValue, newValue);
	}

	public void removeFixedSizeChangeListener(final PropertyChangeListener listener) {
		listeners.removePropertyChangeListener(listener);
	}

	public int getFixedSize() {
		return fixedSize;
	}

	public void setFixedSize(final int newSize) {
		if (newSize == fixedSize) {
			return;
		}

		final int oldSize = fixedSize;
		fixedSize = newSize;
		firePropertyChange(oldSize, fixedSize);
	}

	class SashPainter implements Listener {
		@Override
		public void handleEvent(final Event e) {
			paint((Sash) e.widget, e.gc);
		}
	}

	Splitter(final Composite parent, final int style) {
		super(parent, checkStyle(style));

		if ((style & SWT.HORIZONTAL) != 0) {
			orientation = SWT.HORIZONTAL;
		}

		addListener(SWT.Resize, e -> layout(true));

		sashListener = this::onDragSash;
	}

	private static int checkStyle(final int style) {
		final int mask = SWT.BORDER;
		return style & mask;
	}

	@Override
	public Point computeSize(final int wHint, final int hHint, final boolean changed) {

		final Control[] localControls = getControls(true);
		if (localControls.length == 0) {
			return new Point(wHint, hHint);
		}

		int width = 0;
		int height = 0;
		final boolean vertical = (getStyle() & SWT.VERTICAL) != 0;
		if (vertical) {
			width = wHint;
			height += (localControls.length - 1) * getSashWidth();
		} else {
			height = hHint;
			width += localControls.length * getSashWidth();
		}
		for (final Control localControl : localControls) {
			if (vertical) {
				final Point size = localControl.computeSize(wHint, SWT.DEFAULT);
				height += size.y;
			} else {
				final Point size = localControl.computeSize(SWT.DEFAULT, hHint);
				if (localControl.getData(MAINTAIN_SIZE) != null) {
					size.x = fixedSize;
				}
				width += size.x;
			}
		}
		// if (wHint != SWT.DEFAULT) width = wHint;
		// if (hHint != SWT.DEFAULT) height = hHint;

		return new Point(width, height);
	}

	/**
	 * Answer SWT.HORIZONTAL if the controls in the SashForm are laid out side
	 * by side. Answer SWT.VERTICAL if the controls in the SashForm are laid out
	 * top to bottom.
	 */
	@Override
	public int getOrientation() {
		return orientation;
	}

	public int getSashWidth() {
		return sashWidth;
	}

	/**
	 * Answer the control that currently is maximized in the SashForm. This
	 * value may be null.
	 */
	public Control getMaximizedControl() {
		return maxControl;
	}

	private Control[] getControls(final boolean onlyVisible) {
		final Control[] children = getChildren();
		Control[] localControls = {};
		for (final Control child : children) {
			if (child instanceof Sash || onlyVisible && !child.getVisible()) {
				continue;
			}

			final Control[] newControls = new Control[localControls.length + 1];
			System.arraycopy(localControls, 0, newControls, 0, localControls.length);
			newControls[localControls.length] = child;
			localControls = newControls;
		}
		return localControls;
	}

	@Override
	public void layout(final boolean changed) {
		final Rectangle area = getClientArea();
		if (area.width == 0 || area.height == 0) {
			return;
		}

		final Control[] newControls = getControls(true);
		if (controls.length == 0 && newControls.length == 0) {
			return;
		}
		controls = newControls;

		if (maxControl != null && !maxControl.isDisposed()) {
			for (final Control control : controls) {
				if (control != maxControl) {
					control.setBounds(-200, -200, 0, 0);
				} else {
					control.setBounds(area);
				}
			}
			return;
		}

		// keep just the right number of sashes
		if (sashes.length < controls.length - 1) {
			final Sash[] newSashes = new Sash[controls.length - 1];
			System.arraycopy(sashes, 0, newSashes, 0, sashes.length);
			final int sashOrientation = orientation == SWT.HORIZONTAL ? SWT.VERTICAL : SWT.HORIZONTAL;
			for (int i = sashes.length; i < newSashes.length; i++) {
				newSashes[i] = new Sash(this, sashOrientation);
				newSashes[i].setBackground(ColorConstants.button);
				newSashes[i].addListener(SWT.Paint, new SashPainter());
				newSashes[i].addListener(SWT.Selection, sashListener);
			}
			sashes = newSashes;
		}
		if (sashes.length > controls.length - 1) {
			if (controls.length == 0) {
				for (final Sash element : sashes) {
					element.dispose();
				}
				sashes = new Sash[0];
			} else {
				final Sash[] newSashes = new Sash[controls.length - 1];
				System.arraycopy(sashes, 0, newSashes, 0, newSashes.length);
				for (int i = controls.length - 1; i < sashes.length; i++) {
					sashes[i].dispose();
				}
				sashes = newSashes;
			}
		}

		if (controls.length == 0) {
			return;
		}

		// @TODO
		// This only works if the orientation is horizontal, there are two
		// children and one
		// of them has been set to maintain its size.
		int x = area.x;
		int width;
		for (final Control control : controls) {
			if (control.getData(MAINTAIN_SIZE) != null) {
				width = fixedSize;
				if (width > area.width) {
					width = area.width - getSashWidth();
				}
				control.setBounds(x, area.y, width, area.height);
				x += width + getSashWidth();
			} else {
				width = Math.max(area.width - fixedSize - getSashWidth(), 0);
				control.setBounds(x, area.y, width, area.height);
				x += width + getSashWidth();
			}
		}
		if (sashes.length > 0) {
			sashes[0].setBounds(controls[0].getBounds().x + controls[0].getBounds().width, area.y, getSashWidth(),
					area.height);
		}
	}

	public void maintainSize(final Control c) {
		final Control[] internalControls = getControls(false);
		for (final Control ctrl : internalControls) {
			if (ctrl == c) {
				ctrl.setData(MAINTAIN_SIZE, true);
				break;
			}
		}
	}

	void paint(final Sash sash, final GC gc) {
		if (getSashWidth() == 0) {
			return;
		}
		final Point size = sash.getSize();
		if (getOrientation() == SWT.HORIZONTAL) {
			gc.setForeground(ColorConstants.buttonDarker);
			gc.drawLine(getSashWidth() - 1, 0, getSashWidth() - 1, size.y);
			gc.setForeground(ColorConstants.buttonLightest);
			gc.drawLine(0, 0, 0, size.y);
		} else {
			gc.setForeground(ColorConstants.buttonDarker);
			gc.drawLine(0, 0, size.x, 0);
			gc.drawLine(0, getSashWidth() - 1, size.x, getSashWidth() - 1);
			gc.setForeground(ColorConstants.buttonLightest);
			gc.drawLine(0, 1, size.x, 1);
		}
	}

	private void onDragSash(final Event event) {
		if (event.detail == SWT.DRAG) {
			// constrain feedback
			final Rectangle area = getClientArea();
			if (orientation == SWT.HORIZONTAL) {
				if (controls[0].getData(MAINTAIN_SIZE) != null) {
					event.x = Math.max(event.x, DRAG_MINIMUM);
				} else {
					event.x = Math.min(event.x, area.width - DRAG_MINIMUM - getSashWidth());
				}
			} else {
				event.y = Math.min(event.y, area.height - DRAG_MINIMUM - getSashWidth());
			}
			return;
		}

		final Sash sash = (Sash) event.widget;
		int sashIndex = -1;
		for (int i = 0; i < sashes.length; i++) {
			if (sashes[i] == sash) {
				sashIndex = i;
				break;
			}
		}
		if (sashIndex == -1) {
			return;
		}

		final Control c1 = controls[sashIndex];
		final Control c2 = controls[sashIndex + 1];
		final Rectangle b1 = c1.getBounds();
		final Rectangle b2 = c2.getBounds();
		controls = getControls(false);

		final Rectangle sashBounds = sash.getBounds();
		if (orientation == SWT.HORIZONTAL) {
			final int shift = event.x - sashBounds.x;
			b1.width += shift;
			b2.x += shift;
			b2.width -= shift;
		} else {
			final int shift = event.y - sashBounds.y;
			b1.height += shift;
			b2.y += shift;
			b2.height -= shift;
		}

		c1.setBounds(b1);
		sash.setBounds(event.x, event.y, event.width, event.height);
		c2.setBounds(b2);
		// @TODO
		// This will only work when you have two controls, one of whom is set to
		// maintain size
		if (c1.getData(MAINTAIN_SIZE) != null) {
			setFixedSize(c1.getBounds().width);
		} else {
			setFixedSize(c2.getBounds().width);
		}
	}

	/**
	 * If orientation is SWT.HORIZONTAL, lay the controls in the SashForm out
	 * side by side. If orientation is SWT.VERTICAL, lay the controls in the
	 * SashForm out top to bottom.
	 */
	@Override
	public void setOrientation(final int orientation) {
		if (this.orientation == orientation) {
			return;
		}
		if (orientation != SWT.HORIZONTAL && orientation != SWT.VERTICAL) {
			SWT.error(SWT.ERROR_INVALID_ARGUMENT);
		}
		this.orientation = orientation;

		final int sashOrientation = orientation == SWT.HORIZONTAL ? SWT.VERTICAL : SWT.HORIZONTAL;
		for (int i = 0; i < sashes.length; i++) {
			sashes[i].dispose();
			sashes[i] = new Sash(this, sashOrientation);
			sashes[i].setBackground(ColorConstants.buttonLightest);
			sashes[i].addListener(SWT.Selection, sashListener);
		}
		layout();
	}

	public void setSashWidth(final int width) {
		sashWidth = width;
	}

	@Override
	public void setLayout(final Layout layout) {
		// SashForm does not use Layout
	}

	/**
	 * Specify the control that should take up the entire client area of the
	 * SashForm. If one control has been maximized, and this method is called
	 * with a different control, the previous control will be minimized and the
	 * new control will be maximized.. if the value of control is null, the
	 * SashForm will minimize all controls and return to the default layout
	 * where all controls are laid out separated by sashes.
	 */
	public void setMaximizedControl(final Control control) {
		if (control == null) {
			if (maxControl != null) {
				maxControl = null;
				layout();
				for (final Sash element : sashes) {
					element.setVisible(true);
				}
			}
			return;
		}

		for (final Sash element : sashes) {
			element.setVisible(false);
		}
		maxControl = control;
		layout();

	}

}
